<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */
class PHPDS_tagger extends PHPDS_dependant
{
	const tag_user = 'user';
	const tag_menu = 'menu';
	const tag_role = 'role';
	const tag_group = 'group';

	/**
	 * Generic setter/getter for the tags.
	 * All parameters must be given explicitly
	 *
	 * As a setter, all 4 params must be given (for the [$object;$target] set the tag name $name to value $value)
	 * As a getter, don't give the value ; either a single value or an array of values will be returned
	 *
	 * @param string $object
	 * @param string $name
	 * @param string $target
	 * @param string $value
	 * @return string|array|nothing
	 */
	public function tag($object, $name, $target, $value = null)
	{
		if (!is_null($value)) {
			return $this->db->invokeQuery('PHPDS_taggerMarkQuery', $object, $name, $target, $value);
		} else {
			return $this->db->invokeQuery('PHPDS_taggerLookupQuery', $object, $name, $target);
		}
	}

	/**
	 * List of [object;target] for the given tag (optionaly restricted to the given $object/$target)
	 *
	 * @param $object
	 * @param $target
	 */
	public function tagList($name, $object, $target = null)
	{
		$result = $this->db->invokeQuery('PHPDS_taggerListQuery', $object, $name, $target);
		if (!is_array($result)) $result = array($result);
		return $result;
	}

	/**
	 * Tag (set/get) the user specified in $target
	 *
	 * @param $name
	 * @param $target
	 * @param $value
	 */
	public function tagUser($name, $target, $value = null)
	{
		return $this->tag(PHPDS_tagger::tag_user, $name, $target, $value);
	}

	/**
	 * Tag (set/get) the current user
	 *
	 * @param $name
	 * @param $value
	 */
	public function tagMe($name, $value = null)
	{
		$me = $this->user->currentUserID();
		if (empty($me)) return false;
		return $this->tag(PHPDS_tagger::tag_user, $name, $me, $value);
	}

	/**
	 * Tag (set/get) the menu specified in $target
	 *
	 * @param $name
	 * @param $target
	 * @param $value
	 */
	public function tagMenu($name, $target, $value = null)
	{
		return $this->tag(PHPDS_tagger::tag_menu, $name, $target, $value);
	}

	/**
	 * Tag (set/get) the current menu
	 *
	 * @param $name
	 * @param $value
	 */
	public function tagHere($name, $value = null)
	{
		$here = $this->navigation->currentMenuID();
		if (empty($here)) return false;
		return $this->tag(PHPDS_tagger::tag_menu, $name, $here, $value);
	}

	/**
	 * Tag (set/get) the role specified in $target
	 *
	 * @param $name
	 * @param $target
	 * @param $value
	 */
	public function tagRole($name, $target, $value = null)
	{
		return $this->tag(PHPDS_tagger::tag_role, $name, $target, $value);
	}

	/**
	 * Tag (set/get) the group specified in $target
	 *
	 * @param $name
	 * @param $target
	 * @param $value
	 */
	public function tagGroup($name, $target, $value = null)
	{
		return $this->tag(PHPDS_tagger::tag_group, $name, $target, $value);
	}

	/**
	 * This function creates tag list which allows a comma separated list of tags.
	 *
	 * @param string $object
	 * @param string $target
	 * @param string $value
	 * @return string|nothing
	 */
	public function tagArea($object, $target, $value = null)
	{
		// Define
		$security = $this->security;

		// Check if we should continue.
		if (empty($target))
			return '';

		// Define.
		$tagnames = '';

		// Check if tags are saved.
		if (! empty($security->post)) {
			if (empty($security->post['tagger'])) {
				$names = '';
			} else {
				$names = $security->post['tagger'];
			}
			$this->db->invokeQuery('PHPDS_updateTagsQuery', $object, $target, $value, $names);
		}

		// Find all related tags.
		$taglist = $this->db->invokeQuery('PHPDS_taggerListTargetQuery', $target, $object);

		if (! empty($taglist)) {
			asort($taglist);
			foreach ($taglist as $tag) {
				$tagname = trim($tag['tagName']);
				$tagnames .= "$tagname,";
			}
			$tagnames = rtrim($tagnames, ",");
		}
		return $tagnames;
	}
}