<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Contains methods to handle standard logins.
 * @author Jason Schoeman
 */
class StandardLogin extends PHPDS_dependant
{
	/**
	 * The original login page.
	 * @var int
	 */
	public $loginPageId = '3682403894';

	/**
	 * The original registration page.
	 * @var int
	 */
	public $registrationPageId = '3727066128';

	/**
	 * The original lost password page.
	 * @var int
	 */
	public $lostPasswordPageId = '1901799184';

	/**
	 * Search the database for the given credentials from a persistent cookie
	 *
	 * @param string $cookie
	 * @return array or false the user record
	 * @date 20100702 (v1.0.0) (ross) created from the lookup_user fct by jason and greg
	 * @version 1.0.0
	 * @author jason, greg, ross
	 */
	public function lookupCookieLogin($cookie)
	{
		$id_crypt = substr($cookie, 0, 6);
		$pass_crypt = substr($cookie, 6, 32);

		$found = false;

		// Fetch all instances which match the id crypt
		$persistent_array = $this->selectCookie($id_crypt);

		// Loop and check for matching pass crypt
		if (!empty($persistent_array)) {
			$persistent_item = end($persistent_array);
			if ($pass_crypt == $persistent_item['pass_crypt']) {
				$cookie_id = $persistent_item['cookie_id'];
				$user_id = $persistent_item['user_id'];
				$found = true;
			}
		}

		// Return false or get the data
		if (!empty($found)) {
			// Delete the old cookie
			$this->deleteCookie($cookie_id);

			// Set a new persistent cookie for the next login
			$this->setUserCookie($user_id);

			// Create login.
			$user_array = $this->selectUserFromCookie($user_id);
			$this->setLogin($user_array, "Persistent Login");
		} else {
			$this->setGuest();
			return false;
		}
	}

	/**
	 * Selects user details from provided cookie.
	 * 
	 * @param varchar $cookie
	 * @return array
	 */
	public function selectUserFromCookie ($cookie)
	{
		return $this->db->invokeQuery('LOGIN_selectUserPersistentQuery', $cookie);
	}

	/**
	 * Select cookie data by providing cookie crypt key.
	 *
	 * @param varchar $id_crypt
	 * @return array
	 */
	public function selectCookie ($id_crypt)
	{
		return $this->db->invokeQuery('LOGIN_selectCookieQuery', $id_crypt);
	}

	/**
	 * Set a persistent cookie to be used as a remember me function
	 *
	 * @param int $user_id
	 * @return array or false the user record
	 * @date 20100702 (v1.0.0) (ross) created function
	 * @version	1.0.0
	 * @author ross
	 */
	public function setUserCookie($user_id)
	{
		return $this->db->invokeQuery('LOGIN_setPersistentCookieQuery', $user_id);
	}

	/**
	 * Delete cookie from database.
	 *
	 * @param int $cookie_id
	 */
	public function deleteCookie($cookie_id)
	{
		$this->db->invokeQuery('LOGIN_deleteCookieQuery', $cookie_id);
	}

	/**
	 * Delete the current persistent cookie from the db and kill the cookie on the user end.
	 *
	 * @return boolean
	 * @date 20100702 (v1.0.0) (ross) created function
	 * @version 1.0.0
	 * @author ross
	 */
	public function clearUserCookie($user_id)
	{
		return $this->db->invokeQuery('LOGIN_deletePersistentCookieQuery', $user_id);
	}

	/**
	 * Loads the username & password html template form.
	 *
	 * @param boolean $return
	 */
	public function loginForm ($return = false)
	{
		$settings = $this->db->essentialSettings;
		$navigation = $this->navigation;
		$security = $this->security;
		$template = $this->template;
		$configuration = $this->configuration;
		$redirect_page = '';

		if (empty($configuration['m']))
			$configuration['m'] == $this->loginPageId;

		// Determine page to post form too.
		if ($configuration['m'] == $this->loginPageId) {
			$post_login_url = $navigation->buildURL($settings['redirect_login']);
		} else {
			$post_login_url = $_SERVER['REQUEST_URI'];
		}

		// Assign reusable username.
		$user_name = (empty($_POST['user_name'])) ? '' : $_POST['user_name'];

		// Determine what to show as per request.
		if (! $this->isLoggedIn()) {
			// Check if not registered link should appear.
			if ((boolean) $settings['allow_registration'] == true) {
				// Check if we have a custom registration page.
				(! empty($settings['registration_page'])) ? $registration = $navigation->buildURL($settings['registration_page']) : $registration = $navigation->buildURL($this->registrationPageId);
				// Create generic button for registration.
				$not_registered_yet = '<a href="' . $registration . '">' . ___('Not registered yet?') . '</a>';
			} else {
				$not_registered_yet = false;
			}
			// Create lost password link.
			$lost_password = '<a href="' . $navigation->buildURL($this->lostPasswordPageId) . '">' . ___('Lost Password?') . '</a>';

			// Create the "remember me" checkbox, if needed
			if (! empty($settings['allow_remember'])) {
				$remember = '<label>' . ___('Remember Me?') . '<input tabindex="3" type="checkbox" name="user_remember" value="remember"></label>';
			} else {
				$remember = '';
			}
			// Create HTML login field.
			return $template->modPrint('login_form', $post_login_url, ___('Username or Email'), ___('Password'), $redirect_page, $lost_password, $not_registered_yet, $remember, $security->postValidation(), ___('Log In'), $user_name);
		}
	}

	/**
	 * Check what to do with login action.
	 *
	 * @verion 1.0.0
	 * @date 2011-06-20
	 * @author Jason Schoeman
	 */
	public function controlLogin()
	{		
		if (! empty($this->configuration['allow_remember']) && empty($_SESSION['user_id']) && isset($_COOKIE['pdspc']) && empty($_REQUEST['logout']) && empty($_POST['login'])) {
			$this->lookupCookieLogin($_COOKIE['pdspc']);
		} else
		// Controls the user logout
		if (! empty($_REQUEST['logout']) && $this->isLoggedIn()) {
			// Call logout script.
			$this->clearLogin(true);
			$this->setGuest();
		} else
		// Controls the user login
		if (! empty($_POST['login'])) {
			if (empty($_POST['user_name']) || empty($_POST['user_password'])) {
				$this->template->loginMessage = ___('You did not complete required username and password fields.');
			} else {
				if ($this->lookupUsername($_POST['user_name'])) {
					if (! $this->processLogin($_POST['user_name'], $_POST['user_password'])) {
						$this->template->loginMessage = ___('You used a valid username with a <strong>wrong password</strong>. Remember, it is Case Sensitive.');
					}
				} else {
					$this->template->loginMessage = ___('Your <strong>username</strong> could not be found. Remember, it is Case Sensitive.');
				}
			}
		} else {
			$this->setGuest();
		}
	}

	/**
	 * Checks to see if user and password is correct and allowed. Then creates session data accordingly.
	 *
	 * @param string $username
	 * @param string $password
	 * @return boolean
	 * @date 20100204 greg: split into pieces
	 * @version	1.1
	 * @author jason, greg
	 */
	public function processLogin($username, $password)
	{
		// Simple method to lookup user by providing username and password.
		$user_array = $this->lookupUser($username, $password);

		// Check if we have a login to process.
		if (! empty($user_array)) {
			$this->setLogin($user_array);
			if ($this->db->essentialSettings['allow_remember'] && isset($_POST['user_remember'])) {
				$this->setUserCookie($user_array['user_id']);
			}
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Search the database for the given credentials
	 *
	 * @param string $username
	 * @param string $password
	 * @return array or false the user record
	 * @date 20100204 (v1.0) (greg) created from Jason's original fct
	 * @date 20100608 (v1.0.1) (greg) moved to query system
	 * @version 1.0.1
	 * @author jason, greg
	 */
	public function lookupUser($username, $password = '')
	{
		$password = $this->security->hashPassword($password);
		return $this->db->invokeQuery('LOGIN_selectUserQuery', $username, $password);
	}

	/**
	 * Check if the username exists.
	 *
	 * @param string $username
	 * @version 1.0.0
	 * @author jason
	 */
	public function lookupUsername($username)
	{
		return $this->db->invokeQuery('LOGIN_selectUserNameQuery', $username);
	}

	/**
	 * Make the given user the logged in user
	 *
	 * @param array $select_user_array
	 * @return nothing
	 * @date 20100204 greg: created from Jason's original fct
	 * @version	1.0
	 * @author jason, greg
	 */
	public function setLogin($select_user_array, $persistent = false)
	{
		$conf = $this->configuration;
		$db = $this->db;

		$user_name_db = $select_user_array['user_name'];
		$user_user_password_db = $select_user_array['user_password'];
		$user_display_name_db = $select_user_array['user_display_name'];
		$user_email_db = $select_user_array['user_email'];
		$user_id_db = $select_user_array['user_id'];
		$user_group_db = $select_user_array['user_group'];
		$user_role_db = $select_user_array['user_role'];
		$user_group_name_db = $select_user_array['user_group_name'];
		$user_role_name_db = $select_user_array['user_role_name'];
		$user_language_db = $select_user_array['language'];
		$user_region_db = $select_user_array['region'];
		// Check if we have a user timezone.
		if (!empty($select_user_array['user_timezone'])) {
			// If we do, assign it.
			$user_timezone_db = $select_user_array['user_timezone'];
		} else if (!empty($conf['system_timezone'])) {
			// If we dont, user servers timezone.
			$user_timezone_db = $conf['system_timezone'];
		} else {
			// Try and locate correct timezone.
			$user_timezone_db = date_default_timezone_get();
		}
		//if ((! empty($user_display_name_db)) && (! empty($user_id_db)) && (($username === $user_name_db) || ($username === $user_email_db)) && ($password === $user_user_password_db)) {
		$_SESSION['user_display_name'] = $user_display_name_db;
		$_SESSION['user_email'] = $user_email_db;
		$_SESSION['user_id'] = $user_id_db;
		$_SESSION['user_name'] = $user_name_db;
		$_SESSION['user_group'] = $user_group_db;
		$_SESSION['user_role'] = $user_role_db;
		$_SESSION['user_role_name'] = $user_role_name_db;
		$_SESSION['user_group_name'] = $user_group_name_db;
		$_SESSION['user_timezone'] = $user_timezone_db;
		// Check if language is empty.
		if (!empty($user_language_db)) {
			$user_language = $user_language_db;
		} else {
			$user_language = $conf['language'];
		}
		// Check if region is empty.
		if (!empty($user_region_db)) {
			$user_region = $user_region_db;
		} else {
			$user_region = $conf['region'];
		}
		// Assign region and language.
		$_SESSION['user_language'] = $user_language;
		$_SESSION['user_region'] = $user_region;
		// Set user_locale.
		$_SESSION['user_locale'] = $this->core->formatLocale(true, $user_language, $user_region);
		if (! empty($this->configuration['m'])) {
			// Log this login.
			if (! $persistent) {
				$db->logArray[] = array('log_type' => 4, 'user_id' => $user_id_db, 'logged_by' => $user_display_name_db, 'log_description' => ___('Logged-in'));
			} else {
				$db->logArray[] = array('log_type' => 4, 'user_id' => $user_id_db, 'logged_by' => $user_display_name_db, 'log_description' => $persistent);
			}
		}

		// Clear cache.
		$this->db->cacheClear();
	}

	/**
	 * Destroys login session data.
	 *
	 * @author Jason Schoeman
	 */
	public function clearLogin($set_guest = true)
	{
		$db = $this->db;

		$this->clearUserCookie($_SESSION['user_id']);
		// Log this login.
		$db->logArray[] = array('log_type' => 5, 'log_description' => ___('Logged-out'));
		// Unset some Session variables.
		unset($_SESSION['user_email']);
		unset($_SESSION['user_id']);
		unset($_SESSION['user_name']);
		unset($_SESSION['user_group']);
		unset($_SESSION['user_role']);
		unset($_SESSION['user_display_name']);
		unset($_SESSION['user_group_name']);
		unset($_SESSION['user_role_name']);
		unset($_SESSION['user_language']);
		unset($_SESSION['user_timezone']);
		unset($_SESSION['user_region']);
		unset($_SESSION['user_locale']);
		// Clear cache.
		$db->cacheClear();
		// Unset all of the session variables.
		$_SESSION = array();
		// Destroy the session.
		session_destroy();
		// Set guest if required.
		if ($set_guest) $this->setGuest();
	}

	/**
	 * Sets all settings to guest account.
	 *
	 * @date 20100608 (v1.0.1) (greg) moved to query system
	 * @return string
	 * @author Jason Schoeman, greg
	 */
	public function setGuest()
	{
		$conf = $this->configuration;
		$db = $this->db;

		// Check if we need to set the guest.
		if (empty($_SESSION['user_name'])) {
			// Get required settings.
			$settings_array = $db->essentialSettings;

			// Check if we have a timezone to display.
			if (!empty($conf['system_timezone'])) {
				// Assign server timezone.
				$user_timezone = $conf['system_timezone'];
			} else {
				// Try and locate correct timezone.
				$user_timezone = date_default_timezone_get();
			}
			// Set guest settings.
			$_SESSION['user_name'] = 'guest';
			$_SESSION['user_display_name'] = 'Guest User';
			$_SESSION['user_role_name'] = '';
			$_SESSION['user_group_name'] = '';
			$_SESSION['user_group'] = $settings_array['guest_group'];
			$_SESSION['user_role'] = $settings_array['guest_role'];
			$_SESSION['user_email'] = '';
			$_SESSION['user_language'] = $conf['language'];
			$_SESSION['user_region'] = $conf['region'];
			$_SESSION['user_timezone'] = $user_timezone;
			$_SESSION['user_locale'] = $this->core->formatLocale();
			$_SESSION['user_id'] = 0;
		}
	}

	/**
	 * Check is user is logged in, return false if not.
	 *
	 * @return boolean
	 */
	public function isLoggedIn()
	{
		// Do a simple check to see if user is logged in.
		if (empty($_SESSION['user_id'])) {
			return false;
		} else {
			return true;
		}
	}
}